/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cassert>
#include <chrono>  // std::chrono::milliseconds
#include <numeric> // std::accumulate
#include <thread>  // std::this_thread::sleep_for
#include <vector>

#include "aidge/operator/ReLU.hpp"
#include "aidge/utils/Types.h"

#include "aidge/backend/cpu/operator/ReLUImpl.hpp"
#include "aidge/backend/cpu/operator/ReLUImpl_forward_kernels.hpp"

Aidge::NbElts_t Aidge::ReLUImpl_cpu::getNbRequiredProtected(const Aidge::IOIndex_t /*inputIdx*/) const {
    // this implementation can be in-place
    return 0;
}

void Aidge::ReLUImpl_cpu::forward() {
    assert(mOp.getInput(0) && "missing input #0");

    // Find the correct kernel type
    auto kernelFunc = Registrar<ReLUImplForward_cpu>::create({
        mOp.getInput(0)->dataType(),
        mOp.getOutput(0)->dataType()});

    // Call kernel
    kernelFunc(mOp.getInput(0)->size(),
        mOp.getInput(0)->getImpl()->rawPtr(),
        mOp.getOutput(0)->getImpl()->rawPtr());
}
